@@ -25,7 +25,7 @@
 #define __COGL_FRAMEBUFFER_PRIVATE_H
 
 #include "cogl-object-private.h"
-#include "cogl-matrix-stack.h"
+#include "cogl-matrix-stack-private.h"
 #include "cogl-clip-state-private.h"
 #include "cogl-journal-private.h"
 #include "cogl-winsys-private.h"
@@ -53,6 +53,7 @@ typedef struct
   CoglBool need_stencil;
   int samples_per_pixel;
   CoglBool swap_throttled;
+  CoglBool depth_texture_enabled;
 } CoglFramebufferConfig;
 
 /* Flags to pass to _cogl_offscreen_new_to_texture_full */
@@ -104,6 +105,16 @@ typedef enum
   COGL_READ_PIXELS_NO_FLIP = 1L << 30
 } CoglPrivateReadPixelsFlags;
 
+typedef struct
+{
+  int red;
+  int blue;
+  int green;
+  int alpha;
+  int depth;
+  int stencil;
+} CoglFramebufferBits;
+
 struct _CoglFramebuffer
 {
   CoglObject          _parent;
@@ -126,20 +137,14 @@ struct _CoglFramebuffer
   float               viewport_y;
   float               viewport_width;
   float               viewport_height;
+  int                 viewport_age;
+  int                 viewport_age_for_scissor_workaround;
 
   CoglClipState       clip_state;
 
-  CoglBool            dirty_bitmasks;
-  int                 red_bits;
-  int                 blue_bits;
-  int                 green_bits;
-  int                 alpha_bits;
-
   CoglBool            dither_enabled;
   CoglColorMask       color_mask;
 
-  int                 samples_per_pixel;
-
   /* We journal the textured rectangles we want to submit to OpenGL so
    * we have an oppertunity to batch them together into less draw
    * calls. */
@@ -164,6 +169,12 @@ struct _CoglFramebuffer
   int                 clear_clip_x1;
   int                 clear_clip_y1;
   CoglBool            clear_clip_dirty;
+
+  /* driver specific */
+  CoglBool            dirty_bitmasks;
+  CoglFramebufferBits bits;
+
+  int                 samples_per_pixel;
 };
 
 typedef enum {
@@ -190,6 +201,8 @@ struct _CoglOffscreen
   int             texture_level_width;
   int             texture_level_height;
 
+  CoglTexture *depth_texture;
+
   CoglOffscreenAllocateFlags allocation_flags;
 
   /* FIXME: _cogl_offscreen_new_to_texture_full should be made to use
@@ -300,7 +313,7 @@ _cogl_free_framebuffer_stack (GSList *stack);
 CoglOffscreen *
 _cogl_offscreen_new_to_texture_full (CoglTexture *texture,
                                      CoglOffscreenFlags create_flags,
-                                     unsigned int level);
+                                     int level);
 
 /*
  * _cogl_push_framebuffers:
@@ -319,6 +332,8 @@ _cogl_push_framebuffers (CoglFramebuffer *draw_buffer,
 
 /*
  * _cogl_blit_framebuffer:
+ * @src: The source #CoglFramebuffer
+ * @dest: The destination #CoglFramebuffer
  * @src_x: Source x position
  * @src_y: Source y position
  * @dst_x: Destination x position
@@ -363,12 +378,14 @@ _cogl_push_framebuffers (CoglFramebuffer *draw_buffer,
  * a separate function to copy the entire buffer for a given mask.
  */
 void
-_cogl_blit_framebuffer (unsigned int src_x,
-                        unsigned int src_y,
-                        unsigned int dst_x,
-                        unsigned int dst_y,
-                        unsigned int width,
-                        unsigned int height);
+_cogl_blit_framebuffer (CoglFramebuffer *src,
+                        CoglFramebuffer *dest,
+                        int src_x,
+                        int src_y,
+                        int dst_x,
+                        int dst_y,
+                        int width,
+                        int height);
 
 void
 _cogl_framebuffer_push_projection (CoglFramebuffer *framebuffer);
@@ -421,11 +438,52 @@ _cogl_framebuffer_try_creating_gl_fbo (CoglContext *ctx,
                                        int texture_level,
                                        int texture_level_width,
                                        int texture_level_height,
+                                       CoglTexture *depth_texture,
                                        CoglFramebufferConfig *config,
                                        CoglOffscreenAllocateFlags flags,
                                        CoglGLFramebuffer *gl_framebuffer);
 
-void
-_cogl_gl_framebuffer_bind (CoglFramebuffer *framebuffer, GLenum target);
+unsigned long
+_cogl_framebuffer_compare (CoglFramebuffer *a,
+                           CoglFramebuffer *b,
+                           unsigned long state);
+
+static inline CoglMatrixEntry *
+_cogl_framebuffer_get_modelview_entry (CoglFramebuffer *framebuffer)
+{
+  CoglMatrixStack *modelview_stack =
+    _cogl_framebuffer_get_modelview_stack (framebuffer);
+  return modelview_stack->last_entry;
+}
+
+static inline CoglMatrixEntry *
+_cogl_framebuffer_get_projection_entry (CoglFramebuffer *framebuffer)
+{
+  CoglMatrixStack *projection_stack =
+    _cogl_framebuffer_get_projection_stack (framebuffer);
+  return projection_stack->last_entry;
+}
+
+CoglBool
+_cogl_framebuffer_read_pixels_into_bitmap (CoglFramebuffer *framebuffer,
+                                           int x,
+                                           int y,
+                                           CoglReadPixelsFlags source,
+                                           CoglBitmap *bitmap,
+                                           CoglError **error);
+
+/*
+ * _cogl_framebuffer_get_stencil_bits:
+ * @framebuffer: a pointer to a #CoglFramebuffer
+ *
+ * Retrieves the number of stencil bits of @framebuffer
+ *
+ * Return value: the number of bits
+ *
+ * Since: 2.0
+ * Stability: unstable
+ */
+int
+_cogl_framebuffer_get_stencil_bits (CoglFramebuffer *framebuffer);
 
 #endif /* __COGL_FRAMEBUFFER_PRIVATE_H */
