@@ -39,11 +39,14 @@
 #include "cogl-context-private.h"
 #include "cogl-display-private.h"
 #include "cogl-framebuffer-private.h"
+#include "cogl-framebuffer-gl-private.h"
 #include "cogl-onscreen-template-private.h"
 #include "cogl-renderer-private.h"
 #include "cogl-swap-chain-private.h"
+#include "cogl-texture-2d-gl.h"
 #include "cogl-texture-2d-private.h"
 #include "cogl-pipeline-opengl-private.h"
+#include "cogl-error-private.h"
 
 static void _cogl_gles2_context_free (CoglGLES2Context *gles2_context);
 
@@ -87,7 +90,7 @@ enum {
   RESTORE_FB_FROM_ONSCREEN,
 };
 
-GQuark
+uint32_t
 _cogl_gles2_context_error_quark (void)
 {
   return g_quark_from_static_string ("cogl-gles2-context-error-quark");
@@ -424,7 +427,7 @@ transient_bind_read_buffer (CoglGLES2Context *gles2_ctx)
         }
       else
         {
-          _cogl_gl_framebuffer_bind (gles2_ctx->read_buffer,
+          _cogl_framebuffer_gl_bind (gles2_ctx->read_buffer,
                                      0 /* target ignored */);
 
           return RESTORE_FB_FROM_ONSCREEN;
@@ -448,14 +451,14 @@ restore_write_buffer (CoglGLES2Context *gles2_ctx,
     case RESTORE_FB_FROM_ONSCREEN:
 
       /* Note: we can't restore the original write buffer using
-       * _cogl_gl_framebuffer_bind() if it's an offscreen
-       * framebuffer because _cogl_gl_framebuffer_bind() doesn't
+       * _cogl_framebuffer_gl_bind() if it's an offscreen
+       * framebuffer because _cogl_framebuffer_gl_bind() doesn't
        * know about the fbo handle owned by the gles2 context.
        */
       if (cogl_is_offscreen (gles2_ctx->write_buffer))
         gl_bind_framebuffer_wrapper (GL_FRAMEBUFFER, 0);
       else
-        _cogl_gl_framebuffer_bind (gles2_ctx->write_buffer, GL_FRAMEBUFFER);
+        _cogl_framebuffer_gl_bind (gles2_ctx->write_buffer, GL_FRAMEBUFFER);
 
       break;
     case RESTORE_FB_NONE:
@@ -1555,14 +1558,14 @@ free_texture_object_data (CoglGLES2TextureObjectData *data)
 }
 
 CoglGLES2Context *
-cogl_gles2_context_new (CoglContext *ctx, GError **error)
+cogl_gles2_context_new (CoglContext *ctx, CoglError **error)
 {
   CoglGLES2Context *gles2_ctx;
   const CoglWinsysVtable *winsys;
 
   if (!cogl_has_feature (ctx, COGL_FEATURE_ID_GLES2_CONTEXT))
     {
-      g_set_error (error, COGL_GLES2_CONTEXT_ERROR,
+      _cogl_set_error (error, COGL_GLES2_CONTEXT_ERROR,
                    COGL_GLES2_CONTEXT_ERROR_UNSUPPORTED,
                    "Backend doesn't support creating GLES2 contexts");
 
@@ -1682,11 +1685,11 @@ cogl_gles2_context_get_vtable (CoglGLES2Context *gles2_ctx)
 static CoglGLES2Offscreen *
 _cogl_gles2_offscreen_allocate (CoglOffscreen *offscreen,
                                 CoglGLES2Context *gles2_context,
-                                GError **error)
+                                CoglError **error)
 {
   CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (offscreen);
   const CoglWinsysVtable *winsys;
-  GError *internal_error = NULL;
+  CoglError *internal_error = NULL;
   CoglGLES2Offscreen *gles2_offscreen;
 
   if (!framebuffer->allocated &&
@@ -1709,8 +1712,8 @@ _cogl_gles2_offscreen_allocate (CoglOffscreen *offscreen,
     {
       winsys->restore_context (framebuffer->context);
 
-      g_error_free (internal_error);
-      g_set_error (error, COGL_FRAMEBUFFER_ERROR,
+      cogl_error_free (internal_error);
+      _cogl_set_error (error, COGL_FRAMEBUFFER_ERROR,
                    COGL_FRAMEBUFFER_ERROR_ALLOCATE,
                    "Failed to bind gles2 context to create framebuffer");
       return NULL;
@@ -1722,6 +1725,7 @@ _cogl_gles2_offscreen_allocate (CoglOffscreen *offscreen,
                                               offscreen->texture_level,
                                               offscreen->texture_level_width,
                                               offscreen->texture_level_height,
+                                              offscreen->depth_texture,
                                               &COGL_FRAMEBUFFER (offscreen)->config,
                                               offscreen->allocation_flags,
                                               &gles2_offscreen->gl_framebuffer))
@@ -1730,7 +1734,7 @@ _cogl_gles2_offscreen_allocate (CoglOffscreen *offscreen,
 
       g_slice_free (CoglGLES2Offscreen, gles2_offscreen);
 
-      g_set_error (error, COGL_FRAMEBUFFER_ERROR,
+      _cogl_set_error (error, COGL_FRAMEBUFFER_ERROR,
                    COGL_FRAMEBUFFER_ERROR_ALLOCATE,
                    "Failed to create an OpenGL framebuffer object");
       return NULL;
@@ -1761,10 +1765,10 @@ cogl_push_gles2_context (CoglContext *ctx,
                          CoglGLES2Context *gles2_ctx,
                          CoglFramebuffer *read_buffer,
                          CoglFramebuffer *write_buffer,
-                         GError **error)
+                         CoglError **error)
 {
   const CoglWinsysVtable *winsys = ctx->display->renderer->winsys_vtable;
-  GError *internal_error = NULL;
+  CoglError *internal_error = NULL;
 
   _COGL_RETURN_VAL_IF_FAIL (gles2_ctx != NULL, FALSE);
 
@@ -1843,8 +1847,8 @@ cogl_push_gles2_context (CoglContext *ctx,
     {
       winsys->restore_context (ctx);
 
-      g_error_free (internal_error);
-      g_set_error (error, COGL_GLES2_CONTEXT_ERROR,
+      cogl_error_free (internal_error);
+      _cogl_set_error (error, COGL_GLES2_CONTEXT_ERROR,
                    COGL_GLES2_CONTEXT_ERROR_DRIVER,
                    "Driver failed to make GLES2 context current");
       return FALSE;
@@ -1928,7 +1932,7 @@ cogl_gles2_texture_2d_new_from_handle (CoglContext *ctx,
                                        int width,
                                        int height,
                                        CoglPixelFormat internal_format,
-                                       GError **error)
+                                       CoglError **error)
 {
   return cogl_texture_2d_new_from_foreign (ctx,
                                            handle,
