@@ -27,18 +27,21 @@
 #ifndef __COGL_MATRIX_H
 #define __COGL_MATRIX_H
 
-#include <glib.h>
+#ifdef COGL_HAS_GTYPE_SUPPORT
+#include <glib-object.h>
+#endif /* COGL_HAS_GTYPE_SUPPORT */
+
 #include "cogl-types.h"
 
 #ifdef COGL_ENABLE_EXPERIMENTAL_API
 #include "cogl-quaternion.h"
 #endif
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * SECTION:cogl-matrix
- * @short_description: Fuctions for initializing and manipulating 4x4 matrices
+ * @short_description: Functions for initializing and manipulating 4x4 matrices
  *
  * Matrices are used in Cogl to describe affine model-view transforms, texture
  * transforms, and projective transforms. This exposes a utility API that can
@@ -129,9 +132,9 @@ cogl_matrix_init_identity (CoglMatrix *matrix);
 /**
  * cogl_matrix_init_translation:
  * @matrix: A 4x4 transformation matrix
- * @tx x coordinate of the translation vector
- * @ty y coordinate of the translation vector
- * @tz z coordinate of the translation vector
+ * @tx: x coordinate of the translation vector
+ * @ty: y coordinate of the translation vector
+ * @tz: z coordinate of the translation vector
  *
  * Resets matrix to the (tx, ty, tz) translation matrix:
  *
@@ -762,6 +765,17 @@ cogl_matrix_is_identity (const CoglMatrix *matrix);
 void
 cogl_matrix_transpose (CoglMatrix *matrix);
 
+/**
+ * cogl_debug_matrix_print:
+ * @matrix: A #CoglMatrix
+ *
+ * Prints the contents of a #CoglMatrix to stdout.
+ *
+ * Since: 2.0
+ */
+void
+cogl_debug_matrix_print (const CoglMatrix *matrix);
+
 #ifdef _COGL_SUPPORTS_GTYPE_INTEGRATION
 
 #define COGL_GTYPE_TYPE_MATRIX (cogl_gtype_matrix_get_type ())
@@ -769,7 +783,7 @@ cogl_matrix_transpose (CoglMatrix *matrix);
 /**
  * cogl_gtype_matrix_get_type:
  *
- * Returns the GType for the registered "CoglMatrix" boxed type. This
+ * Returns: the GType for the registered "CoglMatrix" boxed type. This
  * can be used for example to define GObject properties that accept a
  * #CoglMatrix value.
  */
@@ -778,7 +792,7 @@ cogl_gtype_matrix_get_type (void);
 
 #endif /* _COGL_SUPPORTS_GTYPE_INTEGRATION */
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_MATRIX_H */
 
