@@ -35,16 +35,25 @@
 #include "cogl-object-private.h"
 #include "cogl-profile.h"
 #include "cogl-queue.h"
-#include "cogl-internal.h"
 #include "cogl-boxed-value.h"
 #include "cogl-pipeline-snippet-private.h"
 #include "cogl-pipeline-state.h"
 #include "cogl-framebuffer.h"
+#include "cogl-bitmask.h"
 
 #include <glib.h>
 
 #ifdef HAVE_COGL_GL
 
+#define COGL_PIPELINE_PROGEND_FIXED_ARBFP 0
+#define COGL_PIPELINE_PROGEND_FIXED       1
+#define COGL_PIPELINE_PROGEND_GLSL        2
+#define COGL_PIPELINE_N_PROGENDS          3
+
+#define COGL_PIPELINE_VERTEND_FIXED 0
+#define COGL_PIPELINE_VERTEND_GLSL  1
+#define COGL_PIPELINE_N_VERTENDS    2
+
 #define COGL_PIPELINE_FRAGEND_ARBFP 0
 #define COGL_PIPELINE_FRAGEND_FIXED 1
 #define COGL_PIPELINE_FRAGEND_GLSL  2
@@ -54,18 +63,32 @@
 
 #ifdef HAVE_COGL_GLES2
 
+#define COGL_PIPELINE_PROGEND_GLSL 0
+#define COGL_PIPELINE_VERTEND_GLSL 0
 #define COGL_PIPELINE_FRAGEND_GLSL 0
+
 #ifdef HAVE_COGL_GLES
+#define COGL_PIPELINE_PROGEND_FIXED 1
+#define COGL_PIPELINE_VERTEND_FIXED 1
 #define COGL_PIPELINE_FRAGEND_FIXED 1
+
+#define COGL_PIPELINE_N_PROGENDS    2
+#define COGL_PIPELINE_N_VERTENDS    2
 #define COGL_PIPELINE_N_FRAGENDS    2
 #else
+#define COGL_PIPELINE_N_PROGENDS    1
+#define COGL_PIPELINE_N_VERTENDS    1
 #define COGL_PIPELINE_N_FRAGENDS    1
 #endif
 
 #else /* HAVE_COGL_GLES2 */
 
 #ifdef HAVE_COGL_GLES
+#define COGL_PIPELINE_PROGEND_FIXED 0
+#define COGL_PIPELINE_VERTEND_FIXED 0
 #define COGL_PIPELINE_FRAGEND_FIXED 0
+#define COGL_PIPELINE_N_PROGENDS    1
+#define COGL_PIPELINE_N_VERTENDS    1
 #define COGL_PIPELINE_N_FRAGENDS    1
 #else
 #error No drivers defined
@@ -75,93 +98,8 @@
 
 #endif /* HAVE_COGL_GL */
 
-#ifdef COGL_PIPELINE_FRAGEND_ARBFP
-#define COGL_PIPELINE_FRAGEND_ARBFP_MASK \
-  (1 << COGL_PIPELINE_FRAGEND_ARBFP)
-#endif
-#ifdef COGL_PIPELINE_FRAGEND_FIXED
-#define COGL_PIPELINE_FRAGEND_FIXED_MASK \
-  (1 << COGL_PIPELINE_FRAGEND_FIXED)
-#endif
-#ifdef COGL_PIPELINE_FRAGEND_GLSL
-#define COGL_PIPELINE_FRAGEND_GLSL_MASK \
-  (1 << COGL_PIPELINE_FRAGEND_GLSL)
-#endif
-
-#define COGL_PIPELINE_FRAGEND_DEFAULT    0
-#define COGL_PIPELINE_FRAGEND_UNDEFINED  3
-
-#ifdef HAVE_COGL_GL
-
-#define COGL_PIPELINE_VERTEND_FIXED 0
-#define COGL_PIPELINE_VERTEND_GLSL  1
-#define COGL_PIPELINE_N_VERTENDS    2
-
-#else /* HAVE_COGL_GL */
-
-#ifdef HAVE_COGL_GLES2
-
-#define COGL_PIPELINE_VERTEND_GLSL  0
-#ifdef HAVE_COGL_GLES
-#define COGL_PIPELINE_VERTEND_FIXED 1
-#define COGL_PIPELINE_N_VERTENDS    2
-#else
-#define COGL_PIPELINE_N_VERTENDS    1
-#endif
-
-#else /* HAVE_COGL_GLES2 */
-
-#ifdef HAVE_COGL_GLES
-#define COGL_PIPELINE_VERTEND_FIXED 0
-#define COGL_PIPELINE_N_VERTENDS    1
-#else
-#error No drivers defined
-#endif /* HAVE_COGL_GLES */
-
-#endif /* HAVE_COGL_GLES2 */
-
-#endif /* HAVE_COGL_GL */
-
-#ifdef COGL_PIPELINE_VERTEND_FIXED
-#define COGL_PIPELINE_VERTEND_FIXED_MASK \
-  (1 << COGL_PIPELINE_VERTEND_FIXED)
-#endif
-#ifdef COGL_PIPELINE_VERTEND_GLSL
-#define COGL_PIPELINE_VERTEND_GLSL_MASK \
-  (1 << COGL_PIPELINE_VERTEND_GLSL)
-#endif
-
-#define COGL_PIPELINE_VERTEND_DEFAULT    0
-#define COGL_PIPELINE_VERTEND_UNDEFINED  3
-
-#define COGL_PIPELINE_VERTEND_DEFAULT    0
-#define COGL_PIPELINE_VERTEND_UNDEFINED  3
-
-/* If we have either of the GLSL backends then we also need a GLSL
-   progend to combine the shaders generated into a single
-   program. Same goes for the fixed progends which are used to flush
-   the matrices */
-#ifdef COGL_PIPELINE_FRAGEND_FIXED
-
-#define COGL_PIPELINE_PROGEND_FIXED      0
-
-#ifdef COGL_PIPELINE_FRAGEND_GLSL
-#define COGL_PIPELINE_PROGEND_GLSL       1
-#define COGL_PIPELINE_N_PROGENDS         2
-#else
-#define COGL_PIPELINE_N_PROGENDS         1
-#endif
-
-#else /* COGL_PIPELINE_FRAGEND_FIXED */
-
-#ifdef COGL_PIPELINE_FRAGEND_GLSL
-#define COGL_PIPELINE_PROGEND_GLSL       0
-#define COGL_PIPELINE_N_PROGENDS         1
-#else
-#define COGL_PIPELINE_N_PROGENDS         0
-#endif
-
-#endif /* COGL_PIPELINE_FRAGEND_FIXED */
+#define COGL_PIPELINE_PROGEND_DEFAULT    0
+#define COGL_PIPELINE_PROGEND_UNDEFINED  3
 
 /* XXX: should I rename these as
  * COGL_PIPELINE_STATE_INDEX_XYZ... ?
@@ -444,11 +382,6 @@ struct _CoglPipeline
    * type to track the tree heirachy so we can share code... */
   CoglNode _parent;
 
-  /* We need to track if a pipeline is referenced in the journal
-   * because we can't allow modification to these pipelines without
-   * flushing the journal first */
-  unsigned long    journal_ref_count;
-
   /* When weak pipelines are destroyed the user is notified via this
    * callback */
   CoglPipelineDestroyCallback destroy_callback;
@@ -457,28 +390,33 @@ struct _CoglPipeline
    * private data is passed to the above callback */
   void *destroy_data;
 
+  /* We need to track if a pipeline is referenced in the journal
+   * because we can't allow modification to these pipelines without
+   * flushing the journal first */
+  unsigned int journal_ref_count;
+
   /* A mask of which sparse state groups are different in this
    * pipeline in comparison to its parent. */
-  unsigned long    differences;
+  unsigned int differences;
 
   /* Whenever a pipeline is modified we increment the age. There's no
    * guarantee that it won't wrap but it can nevertheless be a
    * convenient mechanism to determine when a pipeline has been
    * changed to you can invalidate some some associated cache that
    * depends on the old state. */
-  unsigned long    age;
+  unsigned int age;
 
   /* This is the primary color of the pipeline.
    *
    * This is a sparse property, ref COGL_PIPELINE_STATE_COLOR */
-  CoglColor        color;
+  CoglColor color;
 
   /* A pipeline may be made up with multiple layers used to combine
    * textures together.
    *
    * This is sparse state, ref COGL_PIPELINE_STATE_LAYERS */
-  GList	          *layer_differences;
   unsigned int     n_layers;
+  GList	          *layer_differences;
 
   /* As a basic way to reduce memory usage we divide the pipeline
    * state into two groups; the minimal state modified in 90% of
@@ -540,20 +478,18 @@ struct _CoglPipeline
    * where the pipeline originates from */
   unsigned int          has_static_breadcrumb:1;
 
-  /* There are multiple fragment processing backends for CoglPipeline,
-   * glsl, arbfp and fixed. This identifies the backend being used for
-   * the pipeline and any private state the backend has associated
-   * with the pipeline. */
-  unsigned int          fragend:3;
-  unsigned int          vertend:3;
+  /* There are multiple fragment and vertex processing backends for
+   * CoglPipeline, glsl, arbfp and fixed that are bundled under a
+   * "progend". This identifies the backend being used for the
+   * pipeline. */
+  unsigned int          progend:3;
 };
 
 typedef struct _CoglPipelineFragend
 {
-  CoglBool (*start) (CoglPipeline *pipeline,
-                     int n_layers,
-                     unsigned long pipelines_difference,
-                     int n_tex_coord_attribs);
+  void (*start) (CoglPipeline *pipeline,
+                 int n_layers,
+                 unsigned long pipelines_difference);
   CoglBool (*add_layer) (CoglPipeline *pipeline,
                          CoglPipelineLayer *layer,
                          unsigned long layers_difference);
@@ -572,10 +508,9 @@ typedef struct _CoglPipelineFragend
 
 typedef struct _CoglPipelineVertend
 {
-  CoglBool (*start) (CoglPipeline *pipeline,
-                     int n_layers,
-                     unsigned long pipelines_difference,
-                     int n_tex_coord_attribs);
+  void (*start) (CoglPipeline *pipeline,
+                 int n_layers,
+                 unsigned long pipelines_difference);
   CoglBool (*add_layer) (CoglPipeline *pipeline,
                          CoglPipelineLayer *layer,
                          unsigned long layers_difference,
@@ -593,9 +528,11 @@ typedef struct _CoglPipelineVertend
 
 typedef struct
 {
+  int vertend;
+  int fragend;
+  CoglBool (*start) (CoglPipeline *pipeline);
   void (*end) (CoglPipeline *pipeline,
-               unsigned long pipelines_difference,
-               int n_tex_coord_attribs);
+               unsigned long pipelines_difference);
   void (*pipeline_pre_change_notify) (CoglPipeline *pipeline,
                                       CoglPipelineState change,
                                       const CoglColor *new_color);
@@ -878,10 +815,7 @@ _cogl_pipeline_weak_copy (CoglPipeline *pipeline,
                           void *user_data);
 
 void
-_cogl_pipeline_set_fragend (CoglPipeline *pipeline, int fragend);
-
-void
-_cogl_pipeline_set_vertend (CoglPipeline *pipeline, int vertend);
+_cogl_pipeline_set_progend (CoglPipeline *pipeline, int progend);
 
 CoglPipeline *
 _cogl_pipeline_get_parent (CoglPipeline *pipeline);
@@ -991,6 +925,14 @@ _cogl_pipeline_foreach_layer_internal (CoglPipeline *pipeline,
                                        CoglPipelineInternalLayerCallback callback,
                                        void *user_data);
 
+CoglBool
+_cogl_pipeline_layer_numbers_equal (CoglPipeline *pipeline0,
+                                    CoglPipeline *pipeline1);
+
+CoglBool
+_cogl_pipeline_layer_and_unit_numbers_equal (CoglPipeline *pipeline0,
+                                             CoglPipeline *pipeline1);
+
 CoglBool
 _cogl_pipeline_need_texture_combine_separate
                                     (CoglPipelineLayer *combine_authority);
@@ -1001,26 +943,11 @@ _cogl_pipeline_init_state_hash_functions (void);
 void
 _cogl_pipeline_init_layer_state_hash_functions (void);
 
-void
-_cogl_pipeline_fragend_layer_change_notify (CoglPipeline *owner,
-                                            CoglPipelineLayer *layer,
-                                            CoglPipelineLayerState change);
-
 CoglPipelineLayerState
 _cogl_pipeline_get_layer_state_for_fragment_codegen (CoglContext *context);
 
 CoglPipelineState
 _cogl_pipeline_get_state_for_fragment_codegen (CoglContext *context);
 
-void
-_cogl_pipeline_vertend_layer_change_notify (CoglPipeline *owner,
-                                            CoglPipelineLayer *layer,
-                                            CoglPipelineLayerState change);
-
-void
-_cogl_pipeline_progend_layer_change_notify (CoglPipeline *owner,
-                                            CoglPipelineLayer *layer,
-                                            CoglPipelineLayerState change);
-
 #endif /* __COGL_PIPELINE_PRIVATE_H */
 
