@@ -31,9 +31,8 @@
 #include <cogl/cogl-pipeline.h>
 #include <cogl/cogl-color.h>
 #include <cogl/cogl-depth-state.h>
-#include <glib.h>
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 #ifdef COGL_ENABLE_EXPERIMENTAL_API
 
@@ -385,7 +384,7 @@ cogl_pipeline_get_alpha_test_reference (CoglPipeline *pipeline);
  * @pipeline: A #CoglPipeline object
  * @blend_string: A <link linkend="cogl-Blend-Strings">Cogl blend string</link>
  *   describing the desired blend function.
- * @error: return location for a #GError that may report lack of driver
+ * @error: return location for a #CoglError that may report lack of driver
  *   support if you give separate blend string statements for the alpha
  *   channel and RGB channels since some drivers, or backends such as
  *   GLES 1.1, don't support this feature. May be %NULL, in which case a
@@ -465,7 +464,7 @@ cogl_pipeline_get_alpha_test_reference (CoglPipeline *pipeline);
 CoglBool
 cogl_pipeline_set_blend (CoglPipeline *pipeline,
                          const char   *blend_string,
-                         GError      **error);
+                         CoglError      **error);
 
 /**
  * cogl_pipeline_set_blend_constant:
@@ -619,7 +618,7 @@ cogl_pipeline_set_user_program (CoglPipeline *pipeline,
  * cogl_pipeline_set_depth_state:
  * @pipeline: A #CoglPipeline object
  * @state: A #CoglDepthState struct
- * @error: A #GError to report failures to setup the given @state.
+ * @error: A #CoglError to report failures to setup the given @state.
  *
  * This commits all the depth state configured in @state struct to the
  * given @pipeline. The configuration values are copied into the
@@ -638,12 +637,12 @@ cogl_pipeline_set_user_program (CoglPipeline *pipeline,
 CoglBool
 cogl_pipeline_set_depth_state (CoglPipeline *pipeline,
                                const CoglDepthState *state,
-                               GError **error);
+                               CoglError **error);
 
 /**
  * cogl_pipeline_get_depth_state
  * @pipeline: A #CoglPipeline object
- * @state: A destination #CoglDepthState struct
+ * @state_out: (out): A destination #CoglDepthState struct
  *
  * Retrieves the current depth state configuration for the given
  * @pipeline as previously set using cogl_pipeline_set_depth_state().
@@ -713,6 +712,8 @@ cogl_pipeline_get_cull_face_mode (CoglPipeline *pipeline);
 
 /**
  * cogl_pipeline_set_front_face_winding:
+ * @pipeline: a #CoglPipeline
+ * @front_winding: the winding order
  *
  * The order of the vertices within a primitive specifies whether it
  * is considered to be front or back facing. This function specifies
@@ -730,7 +731,8 @@ cogl_pipeline_set_front_face_winding (CoglPipeline *pipeline,
                                       CoglWinding front_winding);
 
 /**
- * cogl_pipeline_set_front_face_winding:
+ * cogl_pipeline_get_front_face_winding:
+ * @pipeline: a #CoglPipeline
  *
  * The order of the vertices within a primitive specifies whether it
  * is considered to be front or back facing. This function specifies
@@ -740,6 +742,8 @@ cogl_pipeline_set_front_face_winding (CoglPipeline *pipeline,
  * %COGL_WINDING_CLOCKWISE sets them to be clockwise. The default is
  * %COGL_WINDING_COUNTER_CLOCKWISE.
  *
+ * Returns: The @pipeline front face winding
+ *
  * Status: Unstable
  * Since: 2.0
  */
@@ -915,6 +919,6 @@ cogl_pipeline_add_snippet (CoglPipeline *pipeline,
 
 #endif /* COGL_ENABLE_EXPERIMENTAL_API */
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_PIPELINE_STATE_H__ */
