@@ -30,7 +30,6 @@
 #endif
 
 #include "cogl-debug.h"
-#include "cogl-internal.h"
 #include "cogl-context-private.h"
 #include "cogl-object.h"
 
@@ -53,7 +52,7 @@
 
 static void _cogl_pipeline_free (CoglPipeline *tex);
 static void recursively_free_layer_caches (CoglPipeline *pipeline);
-static gboolean _cogl_pipeline_is_weak (CoglPipeline *pipeline);
+static CoglBool _cogl_pipeline_is_weak (CoglPipeline *pipeline);
 
 const CoglPipelineFragend *_cogl_pipeline_fragends[COGL_PIPELINE_N_FRAGENDS];
 const CoglPipelineVertend *_cogl_pipeline_vertends[COGL_PIPELINE_N_VERTENDS];
@@ -79,6 +78,9 @@ _cogl_pipeline_progends[MAX (COGL_PIPELINE_N_PROGENDS, 1)];
 #include "cogl-pipeline-vertend-fixed-private.h"
 #endif
 
+#ifdef COGL_PIPELINE_PROGEND_FIXED_ARBFP
+#include "cogl-pipeline-progend-fixed-arbfp-private.h"
+#endif
 #ifdef COGL_PIPELINE_PROGEND_FIXED
 #include "cogl-pipeline-progend-fixed-private.h"
 #endif
@@ -125,6 +127,10 @@ _cogl_pipeline_init_default_pipeline (void)
   _cogl_pipeline_fragends[COGL_PIPELINE_FRAGEND_FIXED] =
     &_cogl_pipeline_fixed_fragend;
 #endif
+#ifdef COGL_PIPELINE_PROGEND_FIXED
+  _cogl_pipeline_progends[COGL_PIPELINE_PROGEND_FIXED_ARBFP] =
+    &_cogl_pipeline_fixed_arbfp_progend;
+#endif
 #ifdef COGL_PIPELINE_PROGEND_FIXED
   _cogl_pipeline_progends[COGL_PIPELINE_PROGEND_FIXED] =
     &_cogl_pipeline_fixed_progend;
@@ -147,8 +153,7 @@ _cogl_pipeline_init_default_pipeline (void)
 
   pipeline->is_weak = FALSE;
   pipeline->journal_ref_count = 0;
-  pipeline->fragend = COGL_PIPELINE_FRAGEND_UNDEFINED;
-  pipeline->vertend = COGL_PIPELINE_VERTEND_UNDEFINED;
+  pipeline->progend = COGL_PIPELINE_PROGEND_UNDEFINED;
   pipeline->differences = COGL_PIPELINE_STATE_ALL_SPARSE;
 
   pipeline->real_blend_enable = FALSE;
@@ -232,7 +237,7 @@ _cogl_pipeline_unparent (CoglNode *pipeline)
   _cogl_pipeline_node_unparent_real (pipeline);
 }
 
-static gboolean
+static CoglBool
 recursively_free_layer_caches_cb (CoglNode *node,
                                   void *user_data)
 {
@@ -267,7 +272,7 @@ recursively_free_layer_caches (CoglPipeline *pipeline)
 static void
 _cogl_pipeline_set_parent (CoglPipeline *pipeline,
                            CoglPipeline *parent,
-                           gboolean take_strong_reference)
+                           CoglBool take_strong_reference)
 {
   /* Chain up */
   _cogl_pipeline_node_set_parent_real (COGL_NODE (pipeline),
@@ -284,12 +289,17 @@ _cogl_pipeline_set_parent (CoglPipeline *pipeline,
    * that depends on the pipeline's ancestry then it may be notified
    * here...
    */
-  if (pipeline->fragend != COGL_PIPELINE_FRAGEND_UNDEFINED &&
-      _cogl_pipeline_fragends[pipeline->fragend]->pipeline_set_parent_notify)
+  if (pipeline->progend != COGL_PIPELINE_PROGEND_UNDEFINED)
     {
+      const CoglPipelineProgend *progend =
+        _cogl_pipeline_progends[pipeline->progend];
       const CoglPipelineFragend *fragend =
-        _cogl_pipeline_fragends[pipeline->fragend];
-      fragend->pipeline_set_parent_notify (pipeline);
+        _cogl_pipeline_fragends[progend->fragend];
+
+      /* Currently only the fragends ever care about reparenting of
+       * pipelines... */
+      if (fragend->pipeline_set_parent_notify)
+        fragend->pipeline_set_parent_notify (pipeline);
     }
 }
 
@@ -339,7 +349,7 @@ _cogl_pipeline_revert_weak_ancestors (CoglPipeline *strong)
 /* XXX: Always have an eye out for opportunities to lower the cost of
  * cogl_pipeline_copy. */
 static CoglPipeline *
-_cogl_pipeline_copy (CoglPipeline *src, gboolean is_weak)
+_cogl_pipeline_copy (CoglPipeline *src, CoglBool is_weak)
 {
   CoglPipeline *pipeline = g_slice_new (CoglPipeline);
 
@@ -368,9 +378,7 @@ _cogl_pipeline_copy (CoglPipeline *src, gboolean is_weak)
   pipeline->deprecated_get_layers_list = NULL;
   pipeline->deprecated_get_layers_list_dirty = TRUE;
 
-  pipeline->fragend = src->fragend;
-
-  pipeline->vertend = src->vertend;
+  pipeline->progend = src->progend;
 
   pipeline->has_static_breadcrumb = FALSE;
 
@@ -419,7 +427,7 @@ cogl_pipeline_new (CoglContext *context)
   return new;
 }
 
-static gboolean
+static CoglBool
 destroy_weak_children_cb (CoglNode *node,
                           void *user_data)
 {
@@ -495,7 +503,7 @@ _cogl_pipeline_free (CoglPipeline *pipeline)
   g_slice_free (CoglPipeline, pipeline);
 }
 
-gboolean
+CoglBool
 _cogl_pipeline_get_real_blend_enabled (CoglPipeline *pipeline)
 {
   _COGL_RETURN_VAL_IF_FAIL (cogl_is_pipeline (pipeline), FALSE);
@@ -593,7 +601,7 @@ _cogl_pipeline_foreach_layer_internal (CoglPipeline *pipeline,
     _cogl_pipeline_get_authority (pipeline, COGL_PIPELINE_STATE_LAYERS);
   int n_layers;
   int i;
-  gboolean cont;
+  CoglBool cont;
 
   n_layers = authority->n_layers;
   if (n_layers == 0)
@@ -608,13 +616,77 @@ _cogl_pipeline_foreach_layer_internal (CoglPipeline *pipeline,
     }
 }
 
+CoglBool
+_cogl_pipeline_layer_numbers_equal (CoglPipeline *pipeline0,
+                                    CoglPipeline *pipeline1)
+{
+  CoglPipeline *authority0 =
+    _cogl_pipeline_get_authority (pipeline0, COGL_PIPELINE_STATE_LAYERS);
+  CoglPipeline *authority1 =
+    _cogl_pipeline_get_authority (pipeline1, COGL_PIPELINE_STATE_LAYERS);
+  int n_layers = authority0->n_layers;
+  int i;
+
+  if (authority1->n_layers != n_layers)
+    return FALSE;
+
+  _cogl_pipeline_update_layers_cache (authority0);
+  _cogl_pipeline_update_layers_cache (authority1);
+
+  for (i = 0; i < n_layers; i++)
+    {
+      CoglPipelineLayer *layer0 = authority0->layers_cache[i];
+      CoglPipelineLayer *layer1 = authority1->layers_cache[i];
+
+      if (layer0->index != layer1->index)
+        return FALSE;
+    }
+
+  return TRUE;
+}
+
+CoglBool
+_cogl_pipeline_layer_and_unit_numbers_equal (CoglPipeline *pipeline0,
+                                             CoglPipeline *pipeline1)
+{
+  CoglPipeline *authority0 =
+    _cogl_pipeline_get_authority (pipeline0, COGL_PIPELINE_STATE_LAYERS);
+  CoglPipeline *authority1 =
+    _cogl_pipeline_get_authority (pipeline1, COGL_PIPELINE_STATE_LAYERS);
+  int n_layers = authority0->n_layers;
+  int i;
+
+  if (authority1->n_layers != n_layers)
+    return FALSE;
+
+  _cogl_pipeline_update_layers_cache (authority0);
+  _cogl_pipeline_update_layers_cache (authority1);
+
+  for (i = 0; i < n_layers; i++)
+    {
+      CoglPipelineLayer *layer0 = authority0->layers_cache[i];
+      CoglPipelineLayer *layer1 = authority1->layers_cache[i];
+      int unit0, unit1;
+
+      if (layer0->index != layer1->index)
+        return FALSE;
+
+      unit0 = _cogl_pipeline_layer_get_unit_index (layer0);
+      unit1 = _cogl_pipeline_layer_get_unit_index (layer1);
+      if (unit0 != unit1)
+        return FALSE;
+    }
+
+  return TRUE;
+}
+
 typedef struct
 {
   int i;
   int *indices;
 } AppendLayerIndexState;
 
-static gboolean
+static CoglBool
 append_layer_index_cb (CoglPipelineLayer *layer,
                        void *user_data)
 {
@@ -631,7 +703,7 @@ cogl_pipeline_foreach_layer (CoglPipeline *pipeline,
   CoglPipeline *authority =
     _cogl_pipeline_get_authority (pipeline, COGL_PIPELINE_STATE_LAYERS);
   AppendLayerIndexState state;
-  gboolean cont;
+  CoglBool cont;
   int i;
 
   /* XXX: We don't know what the user is going to want to do to the layers
@@ -651,10 +723,10 @@ cogl_pipeline_foreach_layer (CoglPipeline *pipeline,
     cont = callback (pipeline, state.indices[i], user_data);
 }
 
-static gboolean
+static CoglBool
 layer_has_alpha_cb (CoglPipelineLayer *layer, void *data)
 {
-  gboolean *has_alpha = data;
+  CoglBool *has_alpha = data;
   *has_alpha = _cogl_pipeline_layer_has_alpha (layer);
 
   /* return FALSE to stop iterating layers if we find any layer
@@ -668,7 +740,7 @@ layer_has_alpha_cb (CoglPipelineLayer *layer, void *data)
   return !(*has_alpha);
 }
 
-static gboolean
+static CoglBool
 _cogl_pipeline_needs_blending_enabled (CoglPipeline    *pipeline,
                                        unsigned long    changes,
                                        const CoglColor *override_color)
@@ -798,7 +870,7 @@ _cogl_pipeline_needs_blending_enabled (CoglPipeline    *pipeline,
       /* has_alpha tracks the alpha status of the GL_PREVIOUS layer.
        * To start with that's defined by the pipeline color which
        * must be fully opaque if we got this far. */
-      gboolean has_alpha = FALSE;
+      CoglBool has_alpha = FALSE;
       _cogl_pipeline_foreach_layer_internal (pipeline,
                                              layer_has_alpha_cb,
                                              &has_alpha);
@@ -831,15 +903,9 @@ _cogl_pipeline_needs_blending_enabled (CoglPipeline    *pipeline,
 }
 
 void
-_cogl_pipeline_set_fragend (CoglPipeline *pipeline, int fragend)
+_cogl_pipeline_set_progend (CoglPipeline *pipeline, int progend)
 {
-  pipeline->fragend = fragend;
-}
-
-void
-_cogl_pipeline_set_vertend (CoglPipeline *pipeline, int vertend)
-{
-  pipeline->vertend = vertend;
+  pipeline->progend = progend;
 }
 
 static void
@@ -1099,11 +1165,11 @@ _cogl_pipeline_init_multi_property_sparse_state (CoglPipeline *pipeline,
     }
 }
 
-static gboolean
+static CoglBool
 check_if_strong_cb (CoglNode *node, void *user_data)
 {
   CoglPipeline *pipeline = COGL_PIPELINE (node);
-  gboolean *has_strong_child = user_data;
+  CoglBool *has_strong_child = user_data;
 
   if (!_cogl_pipeline_is_weak (pipeline))
     {
@@ -1114,17 +1180,17 @@ check_if_strong_cb (CoglNode *node, void *user_data)
   return TRUE;
 }
 
-static gboolean
+static CoglBool
 has_strong_children (CoglPipeline *pipeline)
 {
-  gboolean has_strong_child = FALSE;
+  CoglBool has_strong_child = FALSE;
   _cogl_pipeline_node_foreach_child (COGL_NODE (pipeline),
                                      check_if_strong_cb,
                                      &has_strong_child);
   return has_strong_child;
 }
 
-static gboolean
+static CoglBool
 _cogl_pipeline_is_weak (CoglPipeline *pipeline)
 {
   if (pipeline->is_weak && !has_strong_children (pipeline))
@@ -1133,7 +1199,7 @@ _cogl_pipeline_is_weak (CoglPipeline *pipeline)
     return FALSE;
 }
 
-static gboolean
+static CoglBool
 reparent_children_cb (CoglNode *node,
                       void *user_data)
 {
@@ -1149,7 +1215,7 @@ void
 _cogl_pipeline_pre_change_notify (CoglPipeline     *pipeline,
                                   CoglPipelineState change,
                                   const CoglColor  *new_color,
-                                  gboolean          from_layer_change)
+                                  CoglBool          from_layer_change)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -1158,7 +1224,7 @@ _cogl_pipeline_pre_change_notify (CoglPipeline     *pipeline,
    * before we can modify it... */
   if (pipeline->journal_ref_count)
     {
-      gboolean skip_journal_flush = FALSE;
+      CoglBool skip_journal_flush = FALSE;
 
       /* XXX: We don't usually need to flush the journal just due to
        * color changes since pipeline colors are logged in the
@@ -1166,11 +1232,11 @@ _cogl_pipeline_pre_change_notify (CoglPipeline     *pipeline,
        * color enables or disables the need for blending. */
       if (change == COGL_PIPELINE_STATE_COLOR)
         {
-          gboolean will_need_blending =
+          CoglBool will_need_blending =
             _cogl_pipeline_needs_blending_enabled (pipeline,
                                                    change,
                                                    new_color);
-          gboolean blend_enable = pipeline->real_blend_enable ? TRUE : FALSE;
+          CoglBool blend_enable = pipeline->real_blend_enable ? TRUE : FALSE;
 
           if (will_need_blending == blend_enable)
             skip_journal_flush = TRUE;
@@ -1185,70 +1251,43 @@ _cogl_pipeline_pre_change_notify (CoglPipeline     *pipeline,
         }
     }
 
-  /* The fixed function backend has no private state and can't
-   * do anything special to handle small pipeline changes so we may as
-   * well try to find a better backend whenever the pipeline changes.
-   *
-   * The programmable backends may be able to cache a lot of the code
-   * they generate and only need to update a small section of that
-   * code in response to a pipeline change therefore we don't want to
-   * try searching for another backend when the pipeline changes.
-   */
-#ifdef COGL_PIPELINE_FRAGEND_FIXED
-  if (pipeline->fragend == COGL_PIPELINE_FRAGEND_FIXED)
-    _cogl_pipeline_set_fragend (pipeline, COGL_PIPELINE_FRAGEND_UNDEFINED);
-#endif
-#ifdef COGL_PIPELINE_VERTEND_FIXED
-  if (pipeline->vertend == COGL_PIPELINE_VERTEND_FIXED)
-    _cogl_pipeline_set_vertend (pipeline, COGL_PIPELINE_VERTEND_UNDEFINED);
-#endif
-
   /* XXX:
    * To simplify things for the vertex, fragment and program backends
    * we are careful about how we report STATE_LAYERS changes.
    *
    * All STATE_LAYERS change notification with the exception of
    * ->n_layers will also result in layer_pre_change_notifications.
-   *  For backends that perform code generation for fragment
-   *  processing they typically need to understand the details of how
-   *  layers get changed to determine if they need to repeat codegen.
-   *  It doesn't help them to
-   * report a pipeline STATE_LAYERS change for all layer changes since
-   * it's so broad, they really need to wait for the specific layer
-   * change to be notified.  What does help though is to report a
-   * STATE_LAYERS change for a change in
-   * ->n_layers because they typically do need to repeat codegen in
-   *  that case.
+   * For backends that perform code generation for fragment processing
+   * they typically need to understand the details of how layers get
+   * changed to determine if they need to repeat codegen.  It doesn't
+   * help them to report a pipeline STATE_LAYERS change for all layer
+   * changes since it's so broad, they really need to wait for the
+   * specific layer change to be notified.  What does help though is
+   * to report a STATE_LAYERS change for a change in ->n_layers
+   * because they typically do need to repeat codegen in that case.
    *
    * Here we ensure that change notifications against a pipeline or
    * against a layer are mutually exclusive as far as fragment, vertex
    * and program backends are concerned.
    */
-  if (!from_layer_change)
+  if (!from_layer_change &&
+      pipeline->progend != COGL_PIPELINE_PROGEND_UNDEFINED)
     {
-      int i;
+      const CoglPipelineProgend *progend =
+        _cogl_pipeline_progends[pipeline->progend];
+      const CoglPipelineVertend *vertend =
+        _cogl_pipeline_vertends[progend->vertend];
+      const CoglPipelineFragend *fragend =
+        _cogl_pipeline_fragends[progend->fragend];
 
-      if (pipeline->fragend != COGL_PIPELINE_FRAGEND_UNDEFINED &&
-          _cogl_pipeline_fragends[pipeline->fragend]->pipeline_pre_change_notify)
-        {
-          const CoglPipelineFragend *fragend =
-            _cogl_pipeline_fragends[pipeline->fragend];
-          fragend->pipeline_pre_change_notify (pipeline, change, new_color);
-        }
+      if (vertend->pipeline_pre_change_notify)
+        vertend->pipeline_pre_change_notify (pipeline, change, new_color);
 
-      if (pipeline->vertend != COGL_PIPELINE_VERTEND_UNDEFINED &&
-          _cogl_pipeline_vertends[pipeline->vertend]->pipeline_pre_change_notify)
-        {
-          const CoglPipelineVertend *vertend =
-            _cogl_pipeline_vertends[pipeline->vertend];
-          vertend->pipeline_pre_change_notify (pipeline, change, new_color);
-        }
+      if (fragend->pipeline_pre_change_notify)
+        fragend->pipeline_pre_change_notify (pipeline, change, new_color);
 
-      for (i = 0; i < COGL_PIPELINE_N_PROGENDS; i++)
-        if (_cogl_pipeline_progends[i]->pipeline_pre_change_notify)
-          _cogl_pipeline_progends[i]->pipeline_pre_change_notify (pipeline,
-                                                                  change,
-                                                                  new_color);
+      if (progend->pipeline_pre_change_notify)
+        progend->pipeline_pre_change_notify (pipeline, change, new_color);
     }
 
   /* There may be an arbitrary tree of descendants of this pipeline;
@@ -1368,7 +1407,7 @@ _cogl_pipeline_pre_change_notify (CoglPipeline     *pipeline,
 void
 _cogl_pipeline_add_layer_difference (CoglPipeline *pipeline,
                                      CoglPipelineLayer *layer,
-                                     gboolean inc_n_layers)
+                                     CoglBool inc_n_layers)
 {
   _COGL_RETURN_IF_FAIL (layer->owner == NULL);
 
@@ -1402,10 +1441,8 @@ _cogl_pipeline_add_layer_difference (CoglPipeline *pipeline,
 void
 _cogl_pipeline_remove_layer_difference (CoglPipeline *pipeline,
                                         CoglPipelineLayer *layer,
-                                        gboolean dec_n_layers)
+                                        CoglBool dec_n_layers)
 {
-  _COGL_RETURN_IF_FAIL (layer->owner == pipeline);
-
   /* - Flush journal primitives referencing the current state.
    * - Make sure the pipeline has no dependants so it may be modified.
    * - If the pipeline isn't currently an authority for the state being
@@ -1421,14 +1458,24 @@ _cogl_pipeline_remove_layer_difference (CoglPipeline *pipeline,
                                     NULL,
                                     !dec_n_layers);
 
-  layer->owner = NULL;
-  cogl_object_unref (layer);
+  /* We only need to remove the layer difference if the pipeline is
+   * currently the owner. If it is not the owner then one of two
+   * things will happen to make sure this layer is replaced. If it is
+   * the last layer being removed then decrementing n_layers will
+   * ensure that the last layer is skipped. If it is any other layer
+   * then the subsequent layers will have been shifted down and cause
+   * it be replaced */
+  if (layer->owner == pipeline)
+    {
+      layer->owner = NULL;
+      cogl_object_unref (layer);
+
+      pipeline->layer_differences =
+        g_list_remove (pipeline->layer_differences, layer);
+    }
 
   pipeline->differences |= COGL_PIPELINE_STATE_LAYERS;
 
-  pipeline->layer_differences =
-    g_list_remove (pipeline->layer_differences, layer);
-
   if (dec_n_layers)
     pipeline->n_layers--;
 }
@@ -1460,7 +1507,7 @@ void
 _cogl_pipeline_update_blend_enable (CoglPipeline *pipeline,
                                     CoglPipelineState change)
 {
-  gboolean blend_enable =
+  CoglBool blend_enable =
     _cogl_pipeline_needs_blending_enabled (pipeline, change, NULL);
 
   if (blend_enable != pipeline->real_blend_enable)
@@ -1487,7 +1534,7 @@ typedef struct
   int first_index_to_prune;
 } CoglPipelinePruneLayersInfo;
 
-static gboolean
+static CoglBool
 update_prune_layers_info_cb (CoglPipelineLayer *layer, void *user_data)
 {
   CoglPipelinePruneLayersInfo *state = user_data;
@@ -1513,17 +1560,21 @@ _cogl_pipeline_prune_to_n_layers (CoglPipeline *pipeline, int n)
   if (authority->n_layers <= n)
     return;
 
-  _cogl_pipeline_pre_change_notify (pipeline,
-                                    COGL_PIPELINE_STATE_LAYERS,
-                                    NULL,
-                                    FALSE);
-
+  /* This call to foreach_layer_internal needs to be done before
+   * calling pre_change_notify because it recreates the layer cache.
+   * We are relying on pre_change_notify to clear the layer cache
+   * before we change the number of layers */
   state.keep_n = n;
   state.current_pos = 0;
   _cogl_pipeline_foreach_layer_internal (pipeline,
                                          update_prune_layers_info_cb,
                                          &state);
 
+  _cogl_pipeline_pre_change_notify (pipeline,
+                                    COGL_PIPELINE_STATE_LAYERS,
+                                    NULL,
+                                    FALSE);
+
   pipeline->differences |= COGL_PIPELINE_STATE_LAYERS;
   pipeline->n_layers = n;
 
@@ -1534,64 +1585,13 @@ _cogl_pipeline_prune_to_n_layers (CoglPipeline *pipeline, int n)
       CoglPipelineLayer *layer = l->data;
       next = l->next; /* we're modifying the list we're iterating */
 
-      if (layer->index > state.first_index_to_prune)
+      if (layer->index >= state.first_index_to_prune)
         _cogl_pipeline_remove_layer_difference (pipeline, layer, FALSE);
     }
 
   pipeline->differences |= COGL_PIPELINE_STATE_LAYERS;
 }
 
-void
-_cogl_pipeline_fragend_layer_change_notify (CoglPipeline *owner,
-                                            CoglPipelineLayer *layer,
-                                            CoglPipelineLayerState change)
-{
-  /* NB: Although layers can have private state associated with them
-   * by multiple backends we know that a layer can't be *changed* if
-   * it has multiple dependants so if we reach here we know we only
-   * have a single owner and can only be associated with a single
-   * backend that needs to be notified of the layer change...
-   */
-  if (owner->fragend != COGL_PIPELINE_FRAGEND_UNDEFINED &&
-      _cogl_pipeline_fragends[owner->fragend]->layer_pre_change_notify)
-    {
-      const CoglPipelineFragend *fragend =
-        _cogl_pipeline_fragends[owner->fragend];
-      fragend->layer_pre_change_notify (owner, layer, change);
-    }
-}
-
-void
-_cogl_pipeline_vertend_layer_change_notify (CoglPipeline *owner,
-                                            CoglPipelineLayer *layer,
-                                            CoglPipelineLayerState change)
-{
-  /* NB: The comment in fragend_layer_change_notify applies here too */
-  if (owner->vertend != COGL_PIPELINE_VERTEND_UNDEFINED &&
-      _cogl_pipeline_vertends[owner->vertend]->layer_pre_change_notify)
-    {
-      const CoglPipelineVertend *vertend =
-        _cogl_pipeline_vertends[owner->vertend];
-      vertend->layer_pre_change_notify (owner, layer, change);
-    }
-}
-
-void
-_cogl_pipeline_progend_layer_change_notify (CoglPipeline *owner,
-                                            CoglPipelineLayer *layer,
-                                            CoglPipelineLayerState change)
-{
-  int i;
-
-  /* Give all of the progends a chance to notice that the layer has
-     changed */
-  for (i = 0; i < COGL_PIPELINE_N_PROGENDS; i++)
-    if (_cogl_pipeline_progends[i]->layer_pre_change_notify)
-      _cogl_pipeline_progends[i]->layer_pre_change_notify (owner,
-                                                           layer,
-                                                           change);
-}
-
 typedef struct
 {
   /* The layer we are trying to find */
@@ -1615,12 +1615,12 @@ typedef struct
   /* When adding a layer we don't need a complete list of
    * layers_to_shift if we find a layer already corresponding to the
    * layer_index.  */
-  gboolean                    ignore_shift_layers_if_found;
+  CoglBool                    ignore_shift_layers_if_found;
 
 } CoglPipelineLayerInfo;
 
 /* Returns TRUE once we know there is nothing more to update */
-static gboolean
+static CoglBool
 update_layer_info (CoglPipelineLayer *layer,
                    CoglPipelineLayerInfo *layer_info)
 {
@@ -1643,7 +1643,7 @@ update_layer_info (CoglPipelineLayer *layer,
 }
 
 /* Returns FALSE to break out of a _foreach_layer () iteration */
-static gboolean
+static CoglBool
 update_layer_info_cb (CoglPipelineLayer *layer,
                       void *user_data)
 {
@@ -1852,7 +1852,7 @@ typedef struct
   unsigned long fallback_layers;
 } CoglPipelineFallbackState;
 
-static gboolean
+static CoglBool
 fallback_layer_cb (CoglPipelineLayer *layer, void *user_data)
 {
   CoglPipelineFallbackState *state = user_data;
@@ -1906,10 +1906,10 @@ fallback_layer_cb (CoglPipelineLayer *layer, void *user_data)
 typedef struct
 {
   CoglPipeline *pipeline;
-  CoglHandle texture;
+  CoglTexture *texture;
 } CoglPipelineOverrideLayerState;
 
-static gboolean
+static CoglBool
 override_layer_texture_cb (CoglPipelineLayer *layer, void *user_data)
 {
   CoglPipelineOverrideLayerState *state = user_data;
@@ -1976,7 +1976,7 @@ _cogl_pipeline_apply_overrides (CoglPipeline *pipeline,
     }
 }
 
-static gboolean
+static CoglBool
 _cogl_pipeline_layers_equal (CoglPipeline *authority0,
                              CoglPipeline *authority1,
                              unsigned long differences,
@@ -2137,7 +2137,7 @@ _cogl_pipeline_resolve_authorities (CoglPipeline *pipeline,
  * COGL_PIPELINE_WRAP_MODE_CLAMP_TO_EDGE because once they get to the
  * journal stage they act exactly the same.
  */
-gboolean
+CoglBool
 _cogl_pipeline_equal (CoglPipeline *pipeline0,
                       CoglPipeline *pipeline1,
                       unsigned long differences,
@@ -2148,7 +2148,7 @@ _cogl_pipeline_equal (CoglPipeline *pipeline0,
   CoglPipeline *authorities0[COGL_PIPELINE_STATE_SPARSE_COUNT];
   CoglPipeline *authorities1[COGL_PIPELINE_STATE_SPARSE_COUNT];
   int bit;
-  gboolean ret;
+  CoglBool ret;
 
   COGL_STATIC_TIMER (pipeline_equal_timer,
                      "Mainloop", /* parent */
@@ -2330,7 +2330,7 @@ _cogl_pipeline_prune_redundant_ancestry (CoglPipeline *pipeline)
 
   if (new_parent != _cogl_pipeline_get_parent (pipeline))
     {
-      gboolean is_weak = _cogl_pipeline_is_weak (pipeline);
+      CoglBool is_weak = _cogl_pipeline_is_weak (pipeline);
       _cogl_pipeline_set_parent (pipeline, new_parent, is_weak ? FALSE : TRUE);
     }
 }
@@ -2364,7 +2364,7 @@ _cogl_pipeline_update_authority (CoglPipeline *pipeline,
     }
 }
 
-gboolean
+CoglBool
 _cogl_pipeline_get_fog_enabled (CoglPipeline *pipeline)
 {
   CoglPipeline *authority;
@@ -2436,7 +2436,7 @@ cogl_pipeline_remove_layer (CoglPipeline *pipeline, int layer_index)
   _cogl_pipeline_update_blend_enable (pipeline, COGL_PIPELINE_STATE_LAYERS);
 }
 
-static gboolean
+static CoglBool
 prepend_layer_to_list_cb (CoglPipelineLayer *layer,
                           void *user_data)
 {
@@ -2572,10 +2572,8 @@ _cogl_pipeline_init_layer_state_hash_functions (void)
     _cogl_pipeline_layer_hash_texture_type_state;
   layer_state_hash_functions[COGL_PIPELINE_LAYER_STATE_TEXTURE_DATA_INDEX] =
     _cogl_pipeline_layer_hash_texture_data_state;
-  layer_state_hash_functions[COGL_PIPELINE_LAYER_STATE_FILTERS_INDEX] =
-    _cogl_pipeline_layer_hash_filters_state;
-  layer_state_hash_functions[COGL_PIPELINE_LAYER_STATE_WRAP_MODES_INDEX] =
-    _cogl_pipeline_layer_hash_wrap_modes_state;
+  layer_state_hash_functions[COGL_PIPELINE_LAYER_STATE_SAMPLER_INDEX] =
+    _cogl_pipeline_layer_hash_sampler_state;
   layer_state_hash_functions[COGL_PIPELINE_LAYER_STATE_COMBINE_INDEX] =
     _cogl_pipeline_layer_hash_combine_state;
   layer_state_hash_functions[COGL_PIPELINE_LAYER_STATE_COMBINE_CONSTANT_INDEX] =
@@ -2592,11 +2590,16 @@ _cogl_pipeline_init_layer_state_hash_functions (void)
   layer_state_hash_functions[_index] =
     _cogl_pipeline_layer_hash_fragment_snippets_state;
 
+  {
   /* So we get a big error if we forget to update this code! */
-  g_assert (COGL_PIPELINE_LAYER_STATE_SPARSE_COUNT == 11);
+  _COGL_STATIC_ASSERT (COGL_PIPELINE_LAYER_STATE_SPARSE_COUNT == 10,
+                       "Don't forget to install a hash function for new "
+                       "pipeline state and update assert at end of "
+                       "_cogl_pipeline_init_state_hash_functions");
+  }
 }
 
-static gboolean
+static CoglBool
 _cogl_pipeline_hash_layer_cb (CoglPipelineLayer *layer,
                               void *user_data)
 {
@@ -2699,8 +2702,13 @@ _cogl_pipeline_init_state_hash_functions (void)
   state_hash_functions[COGL_PIPELINE_STATE_FRAGMENT_SNIPPETS_INDEX] =
     _cogl_pipeline_hash_fragment_snippets_state;
 
+  {
   /* So we get a big error if we forget to update this code! */
-  g_assert (COGL_PIPELINE_STATE_SPARSE_COUNT == 16);
+  _COGL_STATIC_ASSERT (COGL_PIPELINE_STATE_SPARSE_COUNT == 16,
+                       "Make sure to install a hash function for "
+                       "newly added pipeline state and update assert "
+                       "in _cogl_pipeline_init_state_hash_functions");
+  }
 }
 
 unsigned int
@@ -2723,7 +2731,7 @@ _cogl_pipeline_hash (CoglPipeline *pipeline,
 
   if (differences & COGL_PIPELINE_STATE_REAL_BLEND_ENABLE)
     {
-      gboolean enable = pipeline->real_blend_enable;
+      CoglBool enable = pipeline->real_blend_enable;
       state.hash =
         _cogl_util_one_at_a_time_hash (state.hash, &enable, sizeof (enable));
     }
@@ -2762,7 +2770,7 @@ typedef struct
   CoglPipelineLayer **layers;
 } AddLayersToArrayState;
 
-static gboolean
+static CoglBool
 add_layer_to_array_cb (CoglPipelineLayer *layer,
                        void *user_data)
 {
@@ -2879,7 +2887,10 @@ _cogl_pipeline_get_layer_state_for_fragment_codegen (CoglContext *context)
      COGL_PIPELINE_LAYER_STATE_UNIT |
      COGL_PIPELINE_LAYER_STATE_FRAGMENT_SNIPPETS);
 
-  if (context->driver == COGL_DRIVER_GLES2)
+  /* If the driver supports GLSL then we might be using gl_PointCoord
+   * to implement the sprite coords. In that case the generated code
+   * depends on the point sprite state */
+  if (cogl_has_feature (context, COGL_FEATURE_ID_GLSL))
     state |= COGL_PIPELINE_LAYER_STATE_POINT_SPRITE_COORDS;
 
   return state;
@@ -2892,7 +2903,7 @@ _cogl_pipeline_get_state_for_fragment_codegen (CoglContext *context)
                              COGL_PIPELINE_STATE_USER_SHADER |
                              COGL_PIPELINE_STATE_FRAGMENT_SNIPPETS);
 
-  if (context->driver == COGL_DRIVER_GLES2)
+  if (!(context->private_feature_flags & COGL_PRIVATE_FEATURE_ALPHA_TEST))
     state |= COGL_PIPELINE_STATE_ALPHA_FUNC;
 
   return state;
