@@ -55,7 +55,8 @@ typedef enum
   COGL_PRIVATE_FEATURE_BUILTIN_POINT_SIZE_UNIFORM = 1L<<19,
   COGL_PRIVATE_FEATURE_QUERY_TEXTURE_PARAMETERS = 1L<<20,
   COGL_PRIVATE_FEATURE_ALPHA_TEXTURES = 1L<<21,
-  COGL_PRIVATE_FEATURE_TEXTURE_SWIZZLE = 1L<<22
+  COGL_PRIVATE_FEATURE_TEXTURE_SWIZZLE = 1L<<22,
+  COGL_PRIVATE_FEATURE_TEXTURE_MAX_LEVEL = 1L<<23
 } CoglPrivateFeatureFlags;
 
 /* Sometimes when evaluating pipelines, either during comparisons or
