@@ -31,9 +31,7 @@
 #ifndef __COGL_SUB_TEXTURE_H
 #define __COGL_SUB_TEXTURE_H
 
-#include <glib.h>
-
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * SECTION:cogl-sub-texture
@@ -49,10 +47,9 @@ G_BEGIN_DECLS
 #define COGL_SUB_TEXTURE(tex) ((CoglSubTexture *) tex)
 typedef struct _CoglSubTexture CoglSubTexture;
 
-#define cogl_sub_texture_new cogl_sub_texture_new_EXP
 /**
  * cogl_sub_texture_new:
- * @context: A #CoglContext pointer
+ * @ctx: A #CoglContext pointer
  * @parent_texture: The full texture containing a sub-region you want
  *                  to make a #CoglSubTexture from.
  * @sub_x: The top-left x coordinate of the parent region to make
@@ -105,7 +102,6 @@ cogl_sub_texture_new (CoglContext *ctx,
 CoglTexture *
 cogl_sub_texture_get_parent (CoglSubTexture *sub_texture);
 
-#define cogl_is_sub_texture cogl_is_sub_texture_EXP
 /**
  * cogl_is_sub_texture:
  * @object: a #CoglObject
@@ -121,6 +117,6 @@ cogl_sub_texture_get_parent (CoglSubTexture *sub_texture);
 CoglBool
 cogl_is_sub_texture (void *object);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_SUB_TEXTURE_H */
