@@ -35,25 +35,27 @@
 
 struct _CoglTexture2D
 {
-  CoglTexture     _parent;
+  CoglTexture _parent;
 
   /* The internal format of the GL texture represented as a
      CoglPixelFormat */
-  CoglPixelFormat format;
-  /* The internal format of the GL texture represented as a GL enum */
-  GLenum          gl_format;
-  /* The texture object number */
-  GLuint          gl_texture;
-  int             width;
-  int             height;
-  GLenum          min_filter;
-  GLenum          mag_filter;
-  GLint           wrap_mode_s;
-  GLint           wrap_mode_t;
-  CoglBool        auto_mipmap;
-  CoglBool        mipmaps_dirty;
-  CoglBool        is_foreign;
+  CoglPixelFormat internal_format;
 
+  CoglBool auto_mipmap;
+  CoglBool mipmaps_dirty;
+  CoglBool is_foreign;
+
+  /* TODO: factor out these OpenGL specific members into some form
+   * of driver private state. */
+
+  /* The internal format of the GL texture represented as a GL enum */
+  GLenum gl_internal_format;
+  /* The texture object number */
+  GLuint gl_texture;
+  GLenum gl_legacy_texobj_min_filter;
+  GLenum gl_legacy_texobj_mag_filter;
+  GLint gl_legacy_texobj_wrap_mode_s;
+  GLint gl_legacy_texobj_wrap_mode_t;
   CoglTexturePixel first_pixel;
 };
 
@@ -67,9 +69,19 @@ _cogl_egl_texture_2d_new_from_image (CoglContext *ctx,
                                      int height,
                                      CoglPixelFormat format,
                                      EGLImageKHR image,
-                                     GError **error);
+                                     CoglError **error);
 #endif
 
+CoglTexture2D *
+_cogl_texture_2d_create_base (CoglContext *ctx,
+                              int width,
+                              int height,
+                              CoglPixelFormat internal_format);
+
+void
+_cogl_texture_2d_set_auto_mipmap (CoglTexture *tex,
+                                  CoglBool value);
+
 /*
  * _cogl_texture_2d_externally_modified:
  * @texture: A #CoglTexture2D object
@@ -84,23 +96,27 @@ _cogl_texture_2d_externally_modified (CoglTexture *texture);
 /*
  * _cogl_texture_2d_copy_from_framebuffer:
  * @texture: A #CoglTexture2D pointer
- * @dst_x: X-position to store the image within the texture
- * @dst_y: Y-position to store the image within the texture
  * @src_x: X-position to within the framebuffer to read from
  * @src_y: Y-position to within the framebuffer to read from
  * @width: width of the rectangle to copy
  * @height: height of the rectangle to copy
+ * @src_fb: A source #CoglFramebuffer to copy from
+ * @dst_x: X-position to store the image within the texture
+ * @dst_y: Y-position to store the image within the texture
+ * @level: The mipmap level of @texture to copy too
  *
- * This copies a portion of the current read framebuffer into the
+ * This copies a portion of the given @src_fb into the
  * texture.
  */
 void
 _cogl_texture_2d_copy_from_framebuffer (CoglTexture2D *texture,
-                                        int dst_x,
-                                        int dst_y,
                                         int src_x,
                                         int src_y,
                                         int width,
-                                        int height);
+                                        int height,
+                                        CoglFramebuffer *src_fb,
+                                        int dst_x,
+                                        int dst_y,
+                                        int level);
 
 #endif /* __COGL_TEXTURE_2D_PRIVATE_H */
