@@ -60,19 +60,24 @@ struct _CoglTextureVtable
 
   CoglBool is_primitive;
 
+  CoglBool (* allocate) (CoglTexture *tex,
+                         CoglError **error);
+
   /* This should update the specified sub region of the texture with a
      sub region of the given bitmap. The bitmap is not converted
      before being passed so the implementation is expected to call
      _cogl_texture_prepare_for_upload with a suitable destination
      format before uploading */
-  CoglBool (* set_region) (CoglTexture    *tex,
-                           int             src_x,
-                           int             src_y,
-                           int             dst_x,
-                           int             dst_y,
-                           unsigned int    dst_width,
-                           unsigned int    dst_height,
-                           CoglBitmap     *bitmap);
+  CoglBool (* set_region) (CoglTexture *tex,
+                           int src_x,
+                           int src_y,
+                           int dst_x,
+                           int dst_y,
+                           int dst_width,
+                           int dst_height,
+                           int level,
+                           CoglBitmap *bitmap,
+                           CoglError **error);
 
   /* This should copy the image data of the texture into @data. The
      requested format will have been first passed through
@@ -81,7 +86,7 @@ struct _CoglTextureVtable
      be necessary). */
   CoglBool (* get_data) (CoglTexture *tex,
                          CoglPixelFormat format,
-                         unsigned int rowstride,
+                         int rowstride,
                          uint8_t *data);
 
   void (* foreach_sub_texture_in_region) (CoglTexture *tex,
@@ -108,22 +113,22 @@ struct _CoglTextureVtable
                                GLuint *out_gl_handle,
                                GLenum *out_gl_target);
 
-  void (* set_filters) (CoglTexture *tex,
-                        GLenum min_filter,
-                        GLenum mag_filter);
+  /* OpenGL driver specific virtual function */
+  void (* gl_flush_legacy_texobj_filters) (CoglTexture *tex,
+                                           GLenum min_filter,
+                                           GLenum mag_filter);
 
   void (* pre_paint) (CoglTexture *tex, CoglTexturePrePaintFlags flags);
   void (* ensure_non_quad_rendering) (CoglTexture *tex);
 
-  void (* set_wrap_mode_parameters) (CoglTexture *tex,
-                                     GLenum wrap_mode_s,
-                                     GLenum wrap_mode_t,
-                                     GLenum wrap_mode_p);
+  /* OpenGL driver specific virtual function */
+  void (* gl_flush_legacy_texobj_wrap_modes) (CoglTexture *tex,
+                                              GLenum wrap_mode_s,
+                                              GLenum wrap_mode_t,
+                                              GLenum wrap_mode_p);
 
   CoglPixelFormat (* get_format) (CoglTexture *tex);
   GLenum (* get_gl_format) (CoglTexture *tex);
-  int (* get_width) (CoglTexture *tex);
-  int (* get_height) (CoglTexture *tex);
 
   CoglTextureType (* get_type) (CoglTexture *tex);
 
@@ -139,6 +144,10 @@ struct _CoglTexture
   CoglObject _parent;
   CoglContext *context;
   GList *framebuffers;
+  int max_level;
+  int width;
+  int height;
+  CoglBool allocated;
   const CoglTextureVtable *vtable;
 };
 
@@ -172,6 +181,8 @@ struct _CoglTexturePixel
 void
 _cogl_texture_init (CoglTexture *texture,
                     CoglContext *ctx,
+                    int width,
+                    int height,
                     const CoglTextureVtable *vtable);
 
 void
@@ -203,21 +214,6 @@ CoglTransformResult
 _cogl_texture_transform_quad_coords_to_gl (CoglTexture *texture,
                                            float *coords);
 
-GLenum
-_cogl_texture_get_gl_format (CoglTexture *texture);
-
-void
-_cogl_texture_set_wrap_mode_parameters (CoglTexture *texture,
-                                        GLenum wrap_mode_s,
-                                        GLenum wrap_mode_t,
-                                        GLenum wrap_mode_p);
-
-
-void
-_cogl_texture_set_filters (CoglTexture *texture,
-                           GLenum min_filter,
-                           GLenum mag_filter);
-
 void
 _cogl_texture_pre_paint (CoglTexture *texture, CoglTexturePrePaintFlags flags);
 
@@ -232,34 +228,19 @@ _cogl_texture_determine_internal_format (CoglPixelFormat src_format,
                                          CoglPixelFormat dst_format);
 
 /* Utility function to help uploading a bitmap. If the bitmap needs
-   premult conversion then it will be copied and *copied_bitmap will
-   be set to TRUE. Otherwise dst_bmp will be set to a shallow copy of
-   src_bmp. The GLenums needed for uploading are returned */
-
+ * premult conversion then a converted copy will be returned,
+ * otherwise a reference to the original source will be returned.
+ *
+ * The GLenums needed for uploading are returned
+ */
 CoglBitmap *
-_cogl_texture_prepare_for_upload (CoglBitmap      *src_bmp,
-                                  CoglPixelFormat  dst_format,
+_cogl_texture_prepare_for_upload (CoglBitmap *src_bmp,
+                                  CoglPixelFormat dst_format,
                                   CoglPixelFormat *dst_format_out,
-                                  GLenum          *out_glintformat,
-                                  GLenum          *out_glformat,
-                                  GLenum          *out_gltype);
-
-void
-_cogl_texture_prep_gl_alignment_for_pixels_upload (int pixels_rowstride);
-
-void
-_cogl_texture_prep_gl_alignment_for_pixels_download (int bpp,
-                                                     int width,
-                                                     int rowstride);
-
-/* Utility function to use as a fallback for getting the data of any
-   texture via the framebuffer */
-
-CoglBool
-_cogl_texture_draw_and_read (CoglTexture *texture,
-                             CoglBitmap  *target_bmp,
-                             GLuint       target_gl_format,
-                             GLuint       target_gl_type);
+                                  GLenum *out_glintformat,
+                                  GLenum *out_glformat,
+                                  GLenum *out_gltype,
+                                  CoglError **error);
 
 CoglBool
 _cogl_texture_is_foreign (CoglTexture *texture);
@@ -300,4 +281,52 @@ _cogl_texture_spans_foreach_in_region (CoglSpan *x_spans,
 CoglTextureType
 _cogl_texture_get_type (CoglTexture *texture);
 
+CoglBool
+_cogl_texture_set_region (CoglTexture *texture,
+                          int width,
+                          int height,
+                          CoglPixelFormat format,
+                          int rowstride,
+                          const uint8_t *data,
+                          int dst_x,
+                          int dst_y,
+                          int level,
+                          CoglError **error);
+
+CoglTexture *
+_cogl_texture_new_from_bitmap (CoglBitmap *bitmap,
+                               CoglTextureFlags flags,
+                               CoglPixelFormat internal_format,
+                               CoglError **error);
+
+CoglBool
+_cogl_texture_set_region_from_bitmap (CoglTexture *texture,
+                                      int src_x,
+                                      int src_y,
+                                      int width,
+                                      int height,
+                                      CoglBitmap *bmp,
+                                      int dst_x,
+                                      int dst_y,
+                                      int level,
+                                      CoglError **error);
+
+CoglBool
+_cogl_texture_needs_premult_conversion (CoglPixelFormat src_format,
+                                        CoglPixelFormat dst_format);
+
+int
+_cogl_texture_get_n_levels (CoglTexture *texture);
+
+void
+_cogl_texture_get_level_size (CoglTexture *texture,
+                              int level,
+                              int *width,
+                              int *height,
+                              int *depth);
+
+void
+_cogl_texture_set_allocated (CoglTexture *texture,
+                             CoglBool allocated);
+
 #endif /* __COGL_TEXTURE_PRIVATE_H */
