@@ -24,15 +24,13 @@
 #ifndef __COGL_WAYLAND_SERVER_H
 #define __COGL_WAYLAND_SERVER_H
 
-#include <glib.h>
-
 #include <wayland-server.h>
 
 #define __COGL_H_INSIDE__
 #include <cogl/cogl-context.h>
 #include <cogl/cogl-texture-2d.h>
 
-G_BEGIN_DECLS
+COGL_BEGIN_DECLS
 
 /**
  * cogl_wayland_display_set_compositor_display:
@@ -54,7 +52,7 @@ cogl_wayland_display_set_compositor_display (CoglDisplay *display,
  * cogl_wayland_texture_2d_new_from_buffer:
  * @ctx: A #CoglContext
  * @buffer: A Wayland buffer
- * @error: A #GError for exceptions
+ * @error: A #CoglError for exceptions
  *
  * Uploads the given Wayland @buffer to a #CoglTexture2D.
  *
@@ -77,8 +75,8 @@ cogl_wayland_display_set_compositor_display (CoglDisplay *display,
 CoglTexture2D *
 cogl_wayland_texture_2d_new_from_buffer (CoglContext *ctx,
                                          struct wl_buffer *buffer,
-                                         GError **error);
+                                         CoglError **error);
 
-G_END_DECLS
+COGL_END_DECLS
 
 #endif /* __COGL_WAYLAND_SERVER_H */
