@@ -32,7 +32,6 @@
 
 #include "cogl-util.h"
 #include "cogl-object.h"
-#include "cogl-internal.h"
 #include "cogl-context-private.h"
 #include "cogl-journal-private.h"
 #include "cogl-pipeline-private.h"
@@ -272,8 +271,8 @@ _cogl_path_fill_nodes_with_clipped_rectangle (CoglPath *path,
                                               CoglFramebuffer *framebuffer,
                                               CoglPipeline *pipeline)
 {
-  if (!(path->data->context->private_feature_flags &
-        COGL_PRIVATE_FEATURE_STENCIL_BUFFER))
+  /* We need at least three stencil bits to combine clips */
+  if (_cogl_framebuffer_get_stencil_bits (framebuffer) >= 3)
     {
       static CoglBool seen_warning = FALSE;
 
@@ -1426,9 +1425,9 @@ _cogl_path_build_stroke_attribute_buffer (CoglPath *path)
     return;
 
   data->stroke_attribute_buffer =
-    cogl_attribute_buffer_new (data->context,
-                               data->path_nodes->len * sizeof (floatVec2),
-                               NULL);
+    cogl_attribute_buffer_new_with_size (data->context,
+                                         data->path_nodes->len *
+                                         sizeof (floatVec2));
 
   buffer = COGL_BUFFER (data->stroke_attribute_buffer);
   buffer_p = _cogl_buffer_map_for_fill_or_fallback (buffer);
